#!/usr/bin/perl -w
use strict;
use Irssi;
use Time::HiRes qw/time usleep/;
use POSIX qw/ceil floor/;
use utf8;

my $width, my $height;
my $aspect;
my $fontaspect = 8/12;
my $frameCounter = 0;    
my $starttime;
my $time, my $beatTime;
my @lumabuf;
my @overlay;
my $logocolor;

my @ginlogo19x8 = split('',
  'åååååååååååååååååå__/\__ååååååååååååååå'.
  'ååååå______ åååååå\ oo /ååå__________ å'.
  'ååå./  ___/_____. _______./         /.å'.
  'ååå|   \__      |/      /|    7      |å'.
  'ååå|_    \      |        |    |      |å'.
  'åzfr/___________|________|____|______|å'.
  'ååååååååååååååååååååååååååååååååååååååå'.
  'åååååå G - I N C O R P O R A T E D åååå');

my @popelogo40x6 = split('',
'ååå__ ____ _ ______  __ ____ __ _____ååå'.                                        
'ååå\\ \\\\__ \\_   __  \\_\\ \\\\__ \\_ \\\\ _  \\_å'.                                       
'ååå/  _(/  //  (/   //  _(/  //   /___/_'.                                      
'åå/   \\___//   /   //   \\___//    __/ /å'.                                         
'å/____/ååå/_______//____/ååå/___ /   /åå'.                                         
'ååååååååååååååååååååååååååååpope\\___/ååå');

my @zfrlogo26x6 = split('',
'åååå_____åååå_____ååå___åå'.                          
'ååå_\__  \_å_\ _  \__\_ \_'.                           
'åå/  _____//   /___/ _/__/'.                           
'å/   /    /    __/   \  \å'.                          
'/__ /    /____/_ ____/  /å'.                           
'ååå\____/zefyrosåååå/____/');

my @oltzulogo41x6 = split('',
'ååå______åå___å______åååå_____åååååååå_åå'.                    
'åå_\ __  \å\  \\\\____ \_ _\__  \_ ____/ \_'.                    
'åå/  (/   //   ( /    //  _____//   /   /'.                   
'å/   /   //    //    //   /    /   /   /å'.                   
'/_______//_   //____//__ /    /_______/åå'.                   
'åååååååååå\___\ååååååååå\____/oltzuåååååå');  


my @titlelogo_chars = split('',
'                                     .  --  .                                   '.
'                               \'                  `                             '.
'                         .                             .                        '.
'                       .$                               $.                      '.
'                      $                the                $                     '.
'                     &                                     &                    '.
'                    \'       __   ___    ___    ___     __   `                   '.
'                          __\_\ _\_ \_ _\_ \_ _\_ \_ __\_\                      '.
'                         /    /  _/__/_  /__/_  /__//    /                      '.
'                        /    /   \  \  \_  \  \_  \/    /                       '.
'                       /____/____/  /   /  /   /  /____/                        '.
'                           irssi\___\\\\____/ \____/                              '.
'                    .                                       .                   '.
'                     &                                     &                    '.
'                      $.            experiment           .$                     '.
'                        $.                             .$                       '.
'                                                                                '.
'                            `                  \'                                '.
'                                    .  __  .                                    ');

my @titlelogo_attr = qw/[0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [0;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [1;37;40m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;37;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;39;49m [1;39;49m [1;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;33;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [1;33;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [1;30;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m [1;39;49m/;

my @squid_chars_80x26 = split('',
  '::::::::::::::::::::::::::::::::::::::::::::::::::::::::\'\'\'\'::::::::::::::::::::'.
  ':::::::::::::::::::::::::::::::::::::::::::::::::::\'\'...::::....```:::::::::::::'.
  '::::::::::::::::::::::::::::::::::::::::::::::::\'..::::::::::::::::.`:::::::::::'.
  '::::::::::::::::::::::::::::::::::::\'\'\'::::::::\'.:::::::::::::::::::::.`:::::::::'.
  ':::::::::::::::::::::::::::::::::\'.:::. ::::: ::::::::::::::::::::::::.`::::::::'.
  '::::::::::::::::::::::::::::::::: :: :: :::: ::::::::::::::\'\'\'\'\':::::::: :::::::'.
  '::::::::::::::::::::::::::::::::::.` `:. :: .::::::::::::\'.:%%%:.`::::::.`::::::'.
  '::::::::::::::::::::::::::::::::::::\' :: :: :::\'\'\'\'\'::::\'.%\'   `%.`:::::: ::::::'.
  ':::::::::::::::::::::::::::::::::::::`:. : :\'.:%%%:.`:: %% %\'  %% :::::: ::::::'.
  ':::::::::::::::::::::::::::::::::::::: :: :  .%\'   `%.`: $%.  ..%$ :::::\'.::::::'.
  '::::::::::::::::::::::::::::::::::::::: :: : %% %\'  %% ::.`:%%%:\'.\'.:::: :::::::'.
  '::::::::::::::::::::::::::::::::::::::::`:.` $%.  ..%$ ::::.....`\'.:::: ::::::::'.
  '::::::::::::::::::::::::::::::::::::::::::: :.`:%%%:\'.\'.:::::::::::::: :\':::::::'.
  ':::::::::::::::::::::::::::::::::::::::::`:.`\'.: ...`\'.:::::::::::::: \'.\'.::::::'.
  '::::::::::::::::::::::::::\' ..:: ::::::::.`::::\'.`:::::::::::::::::\'.\'::.`::::::'.
  '::::::::::::::::::::::::\'.::::\'.:::::\'\'...`  ..zfr.``::::::::::::\'.:.`.`::.`::::'.
  '::::::::\'::.::`\'\'...``:: :::: :::::\'.::::::::.``\'..:::::\' .:::\':::.``\' .::: ::::'.
  '::::::::::::`:::::\'\'::\':: ::::.....:::::\'\':::::\'\'\'\':::\'.::::\'\'. ``::::::::\'.::::'.
  ':::.:`::::::::.:..::..::::.`:::::::::\'..\'..`\'\'.:::. .. ::::.`:::.`,````\'\'.::::::'.
  '::::::.`\'\'...``::\'\'...``::::..```\'\'..:\'.:::: :::\':::.\'..`::::.`:::.\'\'.::.`::::::'.
  ':.:::::::::::::..::::::::\'.`\'.:::::.\'.:::\'\'..::: `:::.``.:::::..``::::\'`:: :::::'.
  '::::.`\'\'...``::\'\'...``:::.`:::\'\',`:::::\'.:\'.: :::`.`::::::::\'.\'.::. . .\':: :::::'.
  ':::::::::::::..:::`:::::\'::...::::.``\'.:\'.: :\':::: \'.``\'\'\'..`:.`::.`:.`\'\'.::::::'.
  '::::::::::::::::::::.:..::::::::::::::::.`::.::::\'.::...:::\'.\'.` ::: :::::::::::'.
  '::::::::::::.::\'::.```\'\'...``::\'\'...``\':::.``\'\'\'.::.`:::\'\'.::.`:::\'.::::::::::::'.
  '::::::::::::::::::::::::::::::..::::::::::::::::::::::...:::::::...::::::::::::::');

my @squid_attr_80x26 = qw/[1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;37;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;34;40m [1;37;40m [1;37;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [1;35;40m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [0;36;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [0;36;49m [0;36;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [0;34;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [1;34;49m [1;37;49m [1;36;49m [1;36;49m [1;36;49m [1;37;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;36;49m [0;36;49m [1;30;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;36;49m [1;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [1;36;49m [1;36;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;37;49m [1;37;49m [1;37;49m [1;30;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;36;49m [1;36;49m [1;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;36;49m [0;36;49m [0;36;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;39;49m [0;39;49m [0;39;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;37;49m [1;37;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;36;49m [1;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [1;37;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;34;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;36;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;36;49m [1;36;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;36;49m [0;36;49m [0;36;49m [1;37;49m [1;37;49m [1;37;49m [1;37;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [1;37;49m [1;37;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;34;49m [1;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;34;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [0;37;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [1;34;49m [1;30;49m [1;30;49m [0;36;49m [1;34;49m [1;34;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [1;37;49m [0;36;49m [0;36;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;37;49m [1;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [0;34;49m [1;34;49m [1;30;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [0;36;49m [1;37;49m [1;37;49m [0;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;30;49m [1;30;49m [0;36;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;37;49m [0;35;49m [0;35;49m [0;35;49m [1;37;49m [1;36;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;35;49m [1;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;36;49m [0;36;49m [0;36;49m [1;36;49m [1;36;49m [1;36;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;30;49m [1;30;49m [1;30;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;30;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;37;49m [1;37;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;37;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [1;30;49m [1;30;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [1;30;49m [1;30;49m [1;37;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;30;49m [1;37;49m [1;37;49m [1;35;49m [1;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;34;49m [1;34;49m [1;37;49m [1;37;49m [1;35;49m [1;35;49m [0;35;49m [1;35;49m [1;35;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [1;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [1;35;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;35;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [0;35;49m [1;34;49m [0;35;49m [0;36;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;37;49m [1;35;49m [1;35;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [1;35;49m [1;35;49m [1;34;49m [1;37;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;36;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;35;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [1;35;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;36;49m [1;34;49m [1;34;49m [1;37;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;35;49m [1;35;49m [0;35;49m [0;35;49m [0;35;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [1;34;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m [0;39;49m/;


my $PI = 3.14159265359;
my $PI2 = 3.14159265359*2;

my $frameLength = 1/30; # FPS

my $sync;
my $tick;
my $BPM = 110;
my $RBP = 4; # rows per beat
# bell characters :D
my @tune = qw(0 0 0 0
              1 0 0 0 
              1 0 0 0 
              1 0 0 0 
              1 1 1 1 
              1 0 0 0 
              1 0 0 0 
              1 0 0 0 
              1 1 1 1 
              1 0 0 0 
              1 0 0 0 
              1 0 0 0 
              1 1 1 1 
              1 0 0 1 
              1 0 0 0 
              1 0 0 0 
              1 0 0 0 
              1 0 0 1 
              1 0 0 0
              1 0 0 0 
              1 1 1 1 
              1 0 0 1 
              1 0 0 0 
              1 0 0 0 
              1 0 0 0 );

# greets to msqrt, re: http://www.pouet.net/topic.php?which=8056&page=1
my @char_luminance_ramp = split('',
  '@MBHENR#KWXDFPQASUZbdehx*8Gm&04LOVYkpq5Tagns69owz$CIu23Jcfry%1v7l+it[]{}?j|()=~!-/<>\"^_\';,:`. ');
my $lumaRampLength = scalar(@char_luminance_ramp) -1;


# 0 black   1 dred   2 dgreen  3 dyellow   4 dblue   5 dmag  6 dcyan   7 lgrey
# 8 dgrey   9 lred  10 bgreen 11 byellow  12 bblue  13 bmag 14 bcyan  15 white

my $palette; # "pointer" to current palette
my @one2onePalette = qw(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15);
my @grayscalePalette = qw(0 8 7 15);
my @coolbluePalette = qw(8 4 6 12 14 15 7);
my @distortiongreenPalette = qw(8 4 2 6 12 10 14 15 7);
my @amigaPalette = qw(5 6 12 14 13 15 7);
my @lasersgreenPalette = qw(4 12 10 11 15);
my @plasmavomitPalette = qw(5 6 12 14 13 9 1);
my $running;

sub luminance($$$) { # param as below
  # my ($x, $y, $luma) = @_;
  $lumabuf[$_[1]*($width) + $_[0] ] = $_[2];
};

sub clearLuminance() {
  for(my $i = 0; $i < $width*$height; $i++){
    $lumabuf[$i] = 0;
  }
}

sub clearOverlay() {
  for(my $i = 0; $i < $width*$height; $i++){
    $overlay[$i] = 'å';
  }
}

sub overlay($$$) {
  # my ($x, $y, $char_with_color) = @_;
  $overlay[ $_[0] + $_[1]*$width ] = $_[2];
}

sub blit($$$$$) {
  #my ($x, $y, $columns, $rows, $image ) = @_;

  for(my $j = 0; $j < $_[3]; $j++){
    for(my $i = 0; $i < $_[2]; $i++){
      $overlay[ int($_[0] + $i) + int($_[1]+ $j)*$width ] = @{$_[4]}->[ $i + $j * $_[2]  ];  
    }
  }

}

sub clamp($) {
  if($_[0] < 0){
    $_[0] = 0;
  } elsif ($_[0] > 1) {
    $_[0] = 1;
  }
  return $_[0];
};

sub max($$) {
  return $_[0] if ($_[0] > $_[1]);
  return $_[1];
}

sub min($$) {
  return $_[0] if ($_[0] < $_[1]);
  return $_[1];
}

sub vLength($$$){
  #sqrt
  return sqrt($_[0]*$_[0] + $_[1]*$_[1] + $_[2]*$_[2]);
}

sub vDot($$$$$$){
  return $_[0]*$_[3]+$_[1]*$_[4]+$_[2]*$_[5];
}

sub Intro() {
  my $x, my $y;
  my $r, my $g, my $b;
  my $uvX; my $uvY;
  my $sdcX; my $sdcY;

  my $xres = $width; 
  my $yres = $height; 

  for($y = 0; $y < $yres; $y++){
    for($x = 0; $x < $xres; $x++){
      
      $uvX = ($x/$xres); # 0..1
      $uvY = 1 - ($y/$yres); # 0..1

      $sdcX = ($uvX*2) - 1; # -1..1
      $sdcY = ($uvY*2) - 1; # -1..1
      
      my $r = sqrt($sdcX*$sdcX + $sdcY*$sdcY);
      my $phi = atan2($sdcY, $sdcX);

      my $color = ( sin($phi*8 + cos($r*8 - $time) + $time/4 ) )*($r);

      luminance($x, $y, $color   ); 
    }
  }

};

sub Tunnel() {
  my $x, my $y;
  my $r, my $g, my $b;
  my $uvX; my $uvY;
  my $sdcX; my $sdcY;

  my $xres = $width; 
  my $yres = $height; 

  for($y = 0; $y < $yres; $y++){
    for($x = 0; $x < $xres; $x++){
      
      $uvX = ($x/$xres); # 0..1
      $uvY = (1 - ($y/$yres) ); # 0..1

      $sdcX = ($uvX*2) - 1; # -1..1
      $sdcY = ($uvY*2) - 1; # -1..1
      
      $sdcX += sin($time/2)*.5;
      $sdcY -= cos($time/3)*.5;

      my $r = sqrt(($sdcX*$sdcX)*$aspect + $sdcY*$sdcY);
      my $phi = atan2($sdcY, $sdcX);

      my $color = 
            max( 
              ( cos(($r*$r*10)  - $time*8  ) ),
              (cos($phi*4 + sin($time)*10 )*8 - 7) * ($r*$r)
            ) - $r;

      luminance( $x, $y, $color   ); 
    }
  }

};

sub Plasma() {
  my $x, my $y;
  my $r, my $g, my $b;
  my $uvX; my $uvY;
  my $sdcX; my $sdcY;

  my $xres = $width; 
  my $yres = $height; 

  for($y = 0; $y < $yres; $y++){
    for($x = 0; $x < $xres; $x++){
      
      $uvX = ($x/$xres); # 0..1
      $uvY = 1 - ($y/$yres); # 0..1

      $sdcX = ($uvX*2) - 1; # -1..1
      $sdcY = ($uvY*2) - 1; # -1..1

      
      my $sdcY1 = $sdcY + sin($time + cos($time) );
      my $sdcX1 = $sdcX - cos($time + sin(5*$time) );

      my $sdcY2 = $sdcY - sin($time + cos(3*$time) );
      my $sdcX2 = $sdcX + cos($time - cos(2*$time) );
      
      my $r1 = sqrt($sdcX1*$sdcX1 + $sdcY1*$sdcY1);
      my $phi1 = atan2($sdcY1, $sdcX1);

      my $r2 = sqrt($sdcX2*$sdcX2 + $sdcY2*$sdcY2);
      my $phi2 = atan2($sdcY2, $sdcX2);

      my $color = ( sin($r1*5)/2 - cos($r2*5)/2 + 1 )/2 ;

      luminance($x, $y, $color   ); 
    }
  }

};

sub rX($$$$) {
  my $c = cos($_[3]);
  my $s = sin($_[3]);
  return (
    $_[0],
    $c*$_[1] - $s*$_[2],
    $s*$_[1] + $c*$_[2] );
};

sub rY($$$$) {
  my $c = cos($_[3]);
  my $s = sin($_[3]);
  return (
    $c*$_[0] - $s*$_[2],
    $_[1],
    $s*$_[0] + $c*$_[2] );
};

sub rZ($$$$) {
  my $c = cos($_[3]);
  my $s = sin($_[3]);
  return (
      $c*$_[0] - $s*$_[1],
      $s*$_[0] + $c*$_[1],
      $_[2] );
};

sub d($$$){
  
  my ($x, $y, $z) = @_; 
  $z -= .2;
  $x -= .2;
  $y -= .1;
  ($x, $y, $z) = rX($x, $y, $z, $time);
  ($x, $y, $z) = rY($x, $y, $z, $time*.8);
  ($x, $y, $z) = rZ($x, $y, $z, $time*0.1);

  #  vec3 d = abs(p) - b;
  #    return min(max(d.x,max(d.y,d.z)),0.0) +
  #             length(max(d,0.0));
  my $size = .8;
  
  
  # cube
  my $dx = abs($x) - $size;  
  my $dy = abs($y) - $size;  
  my $dz = abs($z) - $size;  
  return min( max($dx, max($dy,$dz)), 0.0) + vLength( max($dx, 0.0), max($dy, 0.0), max($dz, 0.0) );


  # torus
  #return vLength( vLength($x,0,$z)-$size, $y, 0) - $size*.25;


  # sphere
  #return sqrt($x*$x + $y*$y + $z*$z) - 0.8;
};


sub kak(){
  my $x, my $y;
  my $r, my $g, my $b;
  my $uvX; my $uvY;
  my $sdcX; my $sdcY;
  my $pX, my $pY, my $pZ;
  my $nx, my $ny, my $nz;

  my $xres = $width; 
  my $yres = $height; 

  my @str = split('', "Insert scenepoetry here");
  for($x = 0; $x < 23; $x++){
    $overlay[5+$x + 20*$width] = "\e[".$sync.";". $logocolor .";40m".$str[$x];
  }

  for($y = 0; $y < $yres; $y++){
    for($x = 0; $x < $xres; $x++){
      
      $uvX = ($x/$xres); # 0..1
      $uvY = 1 - ($y/$yres); # 0..1

      $sdcX = ($uvX*2) - 1; # -1..1
      $sdcY = ($uvY*2) - 1; # -1..1

      my $lum = .01;

      my $z = 0.0;
      #my $step = 0.1;
      my $dist;
      while ($z <= 1.5){

        my $pX = $sdcX * (1+$z*1.25 );
        my $pY = $sdcY * (1+$z*(1.25/$aspect));
        my $pZ = ($z - 1.0);
        $dist = d($pX, $pY, $pZ);
        
        if($dist < 0.1) {
          my $pXns = $pX+0.02;
          my $pYns = $pY+0.02;

          my $xZns = d($pXns, $pY, $pZ);
          my $yZns = d($pX, $pYns, $pZ);

          $nx = $dist - $xZns;
          $ny = $dist - $yZns;

          ($nx, $ny, $nz) = vCross( 
            0, 1-$yZns, $yZns,
            1-$xZns, 0, $xZns
          );


          my $lx = 1.0;
          my $ly = 0.0;
          my $lz = 0.0;
          my $li = 5;

          #$lum = 1 - 10*vDot($nx,$ny,$nz,$lx,$ly,$lz);
          $lum = 1- $xZns*8 - $yZns;
          last;
        }
        $z += $dist;
      }

      luminance($x, $y, $lum  ); 
    }
  }
};

sub vCross($$$$$$){
  my ($bx,$by,$bz,$cx,$cy,$cz) = @_;
          # B = [ $pXns $pY $xZns ]
          # C = [ $pX $pYns $yZns ]
          #     ax = bycz - bzcy
          #     ay = bzcx - bxcz
          #     az = bxcy - bycx

  return (  $by*$cz - $bz*$cy,
            $bz*$cx - $bx*$cz,
            $bx*$cy - $by*$cx ); 
};

sub twister() {
  my ($radius,$posX,$posY);
  $posX = 20;
  $posY = 12;
  $radius = 50;

  my($x,$y,$theta);

  my $zlimit = 0;
  my $N = 5;
  my $size = 20;
  my @corners;

  for(my $i = 0; $i < $N; $i++){
    my $theta = $i*($PI2/$N) + ($PI2/$N)/2;
    if(cos($theta) < $zlimit) {
      $zlimit = $theta;
    }
    push(@corners, $theta);
  }

  my $yres = $height; 
  my ($i, $p, $s, $c);


  for($y = 0; $y < $yres; $y++){

    my @tmp;
    foreach my $p (@corners){

      push(@tmp,  $p + sin(($y-($time-60)*1.6)*(sin(($time-60)*2)*0.08 + 0.05 )
        ) + $time - 60  );
    }


    for( my $i = 0; $i < $N; $i++){
      $s = sin($tmp[$i]);
      $c = cos($tmp[$i]);
      if($s <= -sin($zlimit) ){
        $x = 50 + $size*$c;

        for(my $n = $x; $n < 50 + cos($tmp[($i+1)% $N])*$size; $n++){
          luminance($n, $y, ($i+1)*(1/($N+1)) -  ((1+$s)/2)*0.05 );
        }
      }
    }
  }
};

sub squid(){
  my $x, my $y;
  my $uvX; my $uvY;
  my $sdcX, my $sdcY;

  my $xoff, my $yoff;
  my $char, my $attr;
  my $xres = $width; 
  my $yres = $height; 

  my $i, my $j;

  for($y = 0; $y < $yres; $y++){
    for($x = 0; $x < $xres; $x++){
      
      $uvX = ($x/$xres); # 0..1
      $uvY = ($y/$yres); # 0..1
    
      $sdcX = ($uvX*2) - 1; # -1..1
      $sdcY = ($uvY*2) - 1; # -1..1

      $yoff = sin($x/8 + $time*2)*0.05;
      $xoff = sin($y/16 + $time)*0.03;

      $i = ( ($uvX+$xoff) * 80.0 ) % 80;
      $j = ( ($uvY+$yoff) * 23.0 ) % 23;

      $char = $squid_chars_80x26[ int($i) + int($j+2)*80 ];
      $attr = $squid_attr_80x26[ int($i) + int($j+2)*80 ];

      if($overlay[$x + $y*$width] eq 'å'){
        overlay($x, $y, "\033".$attr.$char );
      } 
    }
  }

};

sub Title(){
  my $x, my $y;
  my $char, my $attr;
  my $xres = $width; 
  my $yres = $height; 


  my $i, my $j;

  my $l = ($time)/5 * 19;

  for($y = 0; $y < $l; $y++){
    for($x = 0; $x < $xres; $x++){

      $char = $titlelogo_chars[ $x + ($y)*$width ];
      $attr = $titlelogo_attr[ $x + ($y)*$width ];

      overlay($x, $y+2, "\033".$attr.$char );
    }
  }

};

sub draw() {
  my $scr ="\033[2;1h";
  my $x, my $y;

  my $luma;
  my $bright;
  my $fg, my $bg, my $c;

  my $xres = $width; 
  my $yres = $height; 
  
  my $row, my $index;
  my $paletteLength = scalar(@$palette) -1;
  my $char;

  # for each line
  for($y = 0; $y < $height; $y++){ 
    $row = $y*$width;
    # jump to line start ( +2 because title bar and coordinates start from 1,1);
    $scr .= "\033[" . ($y + 2) .";0H";
    
    # draw the whole row
    for($x = 0; $x < $width; $x++){
      $index = $row + $x;

      # check if overlay is empty
      if( $overlay[ $index ] ne 'å' ){

        $char = "\e[".$sync.";". $logocolor .";40m".$overlay[ $index ];
      } else { # if it is, render the luma thing

        #average luma from samples
        $luma = 0;
        $luma = clamp( $lumabuf[ $x + $y*$xres ] );
        
        #get char
        $c = $char_luminance_ramp[ int( (1 - $luma) * ( $lumaRampLength ) ) ]; 

        # get color
        $fg = $$palette[ min(int( ($luma) * ( $paletteLength ) + 1 ),
        $paletteLength) ];  

        $bright = 0;
        if($fg > 7){
          $bright = 1;
          $fg -= 8;
        }

        $fg += 30;
        $bg = 40;

        $char = "\033[$bright;$bg;$fg"."m".$c ;
      }

      $scr .= $char; 
    }   
  }
  syswrite STDERR, $scr;
}

sub kbhit() {my($rin,$nfd);vec($rin,fileno(STDIN),1)=1;return $nfd=select($rin,undef,undef,0);}

sub main() {
  Irssi::command("window new hidden");
  my $window = Irssi::active_win();
  $width = $window->{"width"} ;
  $height = $window->{"height"} ;
  $aspect = ($width*$fontaspect)/$height;
  

  # height 22 bc irssi bars take 3 lines for topicnshit
  if($width < 80 or $height < 22){
    $window->print("Please resize your terminal to 80x25", MSGLEVEL_CLIENTCRAP);
    return;
  }
  
  # force res due to upcoming acii shizzle
  $width = 80;
  $height = 22;
 
  Irssi::command("clear");
  print STDERR "\0337"; #save cursor position & attr
  print STDERR "\033[0m"; #reset all attr;


  clearOverlay();
  clearLuminance();

  my $prevTime, my $frameTime;
  $tick = 0;
  $starttime = time;
  $running = 1;
  while(1){
    $prevTime = $time;
    $time = time - $starttime;
    $frameTime = $time - $prevTime;
    $beatTime += $frameTime;

    if( $frameTime < $frameLength ){
      usleep( ($frameLength - $frameTime)*1000000 );
    }

    if($beatTime > 60/($BPM*$RBP)){
      $beatTime = 0;
      $tick++;
      if($tune[$tick % (scalar(@tune)-1)  ] ){
        print STDERR "\007";
        $sync = 1;
      }
    }
    $frameCounter++;

    demo();
    draw();
    $sync = 0;
    last if( kbhit() or $running != 1 );
  }
  print STDERR "\0338"; #restore cursor pos & attr

  print STDERR "\033c";
  Irssi::command("wc");
  Irssi::command("redraw");
} 


sub Greets(){
  my $size = 23;
  my $greets = "    Greetings to".
  (' 'x($size+7)) ."Hedelmae Ananasmurska  Traction Dekadence FIT Primitive Disaster AreaFaemiyah Blobtrox AdaptMFX Scenesat Trilobit FTITAN Mirage Jumalauta DSS Darklite Alumni                           and everyone @ Payback   ";
  my @str = split('', $greets);

  my $limit = length($greets)*clamp( ($time-50)/5  );
  for(my $i = 0; $i < $limit; $i++){
    my $x = 2 + $i % $size;
    my $y = 5 + int($i / $size);
    my $color = 37;
    my $bright = 0;
    
    $bright = 1 if ( $i < $limit -2 );

    $overlay[$x + $y*$width] = "\e[".$bright.";". $color .";40m".$str[$i];
  }

};

# timeline

sub demo(){
  $palette = \@grayscalePalette;
  if($time < 5){                      # Title part
    clearOverlay();
    clearLuminance();
    Title();
  } elsif ($time < 10) {
    clearOverlay();
    blit( ($width / 2 )-19, -4 + $height/2, 39, 8, \@ginlogo19x8);
    $palette = \@grayscalePalette;
    $logocolor = 31;
    Intro();
  } elsif ($time < 20) {
    clearOverlay();
    clearLuminance();
    $palette = \@coolbluePalette;
    $logocolor = 36;
    blit( 5, 5, 40, 6, \@popelogo40x6);
    Tunnel();
  } elsif ($time < 30) {
    clearOverlay();
    clearLuminance();
    $palette = \@plasmavomitPalette;
    $logocolor = 33;
    blit( 35, 10, 41, 6, \@oltzulogo41x6);
    Plasma(); 
  } elsif ($time < 40) {             # Squid part
    clearOverlay();
    clearLuminance();
    $palette = \@one2onePalette;
    $logocolor = 36;
    blit( 2, 15, 26, 6, \@zfrlogo26x6);
    squid();
  } elsif ($time < 50) { 
    clearOverlay();
    clearLuminance();
    $palette = \@coolbluePalette;
    $logocolor = 37;
    kak();  
  } elsif ($time < 60) {
    clearOverlay();
    clearLuminance();
    $palette = \@plasmavomitPalette;
    Greets();
    twister();
  } else {
    $running = 0;
  }
};

main();
